<?php
// SMSapi COMPONENT
//
// Copyright (c) 2012 NetAware Inc.  All rights reserved.
//  
// $Id:$

require_once(dirname(__FILE__).'/../componenthelper.inc.php');
require_once(dirname(__FILE__).'/smsapi.util.php');
require_once(dirname(__FILE__).'/smsapi.clickatell.php');
require_once(dirname(__FILE__).'/smsapi.twilio.php');

// run the initialization function
smsapi_component_init();

////////////////////////////////////////////////////////////////////////
// COMPONENT INIT FUNCTIONS
////////////////////////////////////////////////////////////////////////

function smsapi_component_init(){

	$component_name="smsapi";
	
	$args=array(

		// need a name
		COMPONENT_NAME => $component_name,
		
		// informative information
		COMPONENT_VERSION => "0.2",
		COMPONENT_DATE => "02-24-2012",
		COMPONENT_AUTHOR => "NetAware Inc",
		COMPONENT_DESCRIPTION => "Provides integration with SMS API gateway services.",
		COMPONENT_TITLE => "SMSapi",
		COMPONENT_COPYRIGHT => "Copyright (c) 2012 NetAware Inc",
		COMPONENT_HOMEPAGE => "http://www.tapgateway.com/",
		COMPONENT_PROTECTED => false,

		// configuration function (optional)
		COMPONENT_CONFIGFUNCTION => "smsapi_component_config_func",
		);
		
	register_component($component_name,$args);
	
		
	// register as a new notification method
	$args=array(
		NOTIFICATIONMETHOD_FUNCTION => 'smsapi_component_notificationmethod_func',
		);
	register_notificationmethod('smsapi',$args);
	
	// register to add a new tab to the notification menus
	register_callback(CALLBACK_USER_NOTIFICATION_METHODS_TABS_INIT,'smsapi_component_methods_addtab');
	register_callback(CALLBACK_USER_NOTIFICATION_MESSAGES_TABS_INIT,'smsapi_component_messages_addtab');	
	}
	


///////////////////////////////////////////////////////////////////////////////////////////
// TAB FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function smsapi_component_messages_addtab($cbtype,&$cbdata){

	// bail if this component has been disabled by the admin
	$settings_raw=get_option("smsapi_component_options");
	if($settings_raw=="")
		$settings=array();
	else
		$settings=unserialize($settings_raw);
	$enabled=grab_array_var($settings,"enabled",0);
	$api=grab_array_var($settings,"api",0);
	if($enabled!=1)
		return;

	$newtab=array(
		"id" => "smsapi",
		"title" => "SMSapi ($api)",
		);

	// add new tab
	$cbdata["tabs"][]=$newtab;	
	}


function smsapi_component_methods_addtab($cbtype,&$cbdata){

	// bail if this component has been disabled by the admin
	$settings_raw=get_option("smsapi_component_options");
	if($settings_raw=="")
		$settings=array();
	else
		$settings=unserialize($settings_raw);
	$enabled=grab_array_var($settings,"enabled",0);
	$api=grab_array_var($settings,"api",0);
	if($enabled!=1)
		return;

	$newtab=array(
		"id" => "smsapi",
		"title" => "SMSapi ($api)",
		);

	// add new tab
	$cbdata["tabs"][]=$newtab;
	}

	
///////////////////////////////////////////////////////////////////////////////////////////
// NOTIFICATION METHOD FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function smsapi_component_notificationmethod_func($mode="",$inargs,&$outargs,&$result){

	// initialize return values
	$result=0;
	$outargs=array();
	$output='';
	
	// bail if this component has been disabled by the admin
	$settings_raw=get_option("smsapi_component_options");
	if($settings_raw=="")
		$settings=array();
	else
		$settings=unserialize($settings_raw);
	$enabled=grab_array_var($settings,"enabled",0);
	if($enabled!=1)
		return $output;

	switch($mode){
	
		case NOTIFICATIONMETHOD_MODE_GETCONFIGOPTIONS:
			// defaults
			$smsapi_notifications_enabled=get_user_meta(0,'smsapi_notifications_enabled');
			$smsapi_mobile_number=get_user_meta(0,'smsapi_mobile_number');
			if($smsapi_mobile_number=="")
				$smsapi_mobile_number=get_user_meta(0,'mobile_number');

			// get values from form submission
			$smsapi_notifications_enabled=grab_request_var("smsapi_notifications_enabled",$smsapi_notifications_enabled);
			$smsapi_mobile_number=grab_request_var("smsapi_mobile_number",$smsapi_mobile_number);
			
			$smsapi_notifications_enabled=checkbox_binary($smsapi_notifications_enabled);			

			$output="
			<table>
			<tr>
			<td valign='top'>
			<input type='checkbox' class='checkbox' name='smsapi_notifications_enabled' ".is_checked($smsapi_notifications_enabled,1)."><br class='nobr' />
			</td>
			<td>
			<b>SMS Text Message</b><br class='nobr' />Receive SMS alerts via SMSapi.<br />
			<table>
				<tr>
				<td><label>Mobile Number:</label></td>
				<td><input type='text' size='15' name='smsapi_mobile_number'value='".$smsapi_mobile_number."' class='textfield' /></td>
				</tr>

			</table>
			</td>
			</tr>
			<tr>
			<td valign='top'>
			<input type='checkbox' class='checkbox' name='smsapi_send_test' ><br class='nobr' />
			</td>
			<td>
			Send a test SMS message to the number specified above.
			</td>
			</tr>
			</table>
			";
			break;
			
		case NOTIFICATIONMETHOD_MODE_SETCONFIGOPTIONS:
			$smsapi_notifications_enabled=grab_array_var($inargs,"smsapi_notifications_enabled",0);
			$smsapi_notifications_enabled=checkbox_binary($smsapi_notifications_enabled);
			$smsapi_mobile_number=grab_array_var($inargs,"smsapi_mobile_number","");

			$smsapi_mobile_number = preg_replace('/[^\d+]/', '', $smsapi_mobile_number);

			// check for errors
			$errors=0;
			$errmsg=array();
			$okmsg=array();
			if($smsapi_notifications_enabled==1){
				if($smsapi_mobile_number==""){
					$errmsg[$errors++]="Mobile phone number for SMSapi alerts is blank.";
					}
				}
		
			// handle errors
			if($errors>0){
				$outargs[NOTIFICATIONMETHOD_ERROR_MESSAGES]=$errmsg;
				$result=1;
				}
				
			// save values
			set_user_meta(0,'smsapi_notifications_enabled',$smsapi_notifications_enabled);
			set_user_meta(0,'smsapi_mobile_number',$smsapi_mobile_number);
			
			// send a test message	
			$testnumber=grab_array_var($inargs,"smsapi_send_test","");
			$tapoutput=array();
			if($testnumber!="" && $errors==0){
				//echo "SENDING TEST...";
				$args=array(
					"number" => $smsapi_mobile_number,
					"message" => "This is a test message from Nagios XI and SMSapi",
					);

				$sendresult=smsapi_component_send($args);
				
				// handle errors
				if($sendresult[success] == 0){
					$errmsg[$errors++]="An error occurred sending the test SMSapi message.";
					$errmsg[]=$sendresult[message];
					$outargs[NOTIFICATIONMETHOD_ERROR_MESSAGES]=$errmsg;
					$result=1;
					//echo "<BR>ERRORS<BR><BR>";
					//print_r($errmsg);
					return '';
					}
					
				// success message
				$okmsg=array();
				if($sendresult[success] == 1){
					$okmsg[]="Test SMSapi message successfully queued for ".$smsapi_mobile_number.".";
					$outargs[NOTIFICATIONMETHOD_INFO_MESSAGES]=$okmsg;
					}					
				}
		
			break;
			
		case NOTIFICATIONMETHOD_MODE_GETMESSAGEFORMAT:
			$component_name="smsapi";

			// defaults/saved values
			$smsapi_notifications_host_message=smsapi_component_get_host_message(0);
			$smsapi_notifications_service_message=smsapi_component_get_service_message(0);
			
			// newly submitted values
			$smsapi_notifications_host_message=grab_array_var($inargs,"smsapi_notifications_host_message",$smsapi_notifications_host_message);
			$smsapi_notifications_service_message=grab_array_var($inargs,"smsapi_notifications_service_message",$smsapi_notifications_service_message);
			
			$component_url=get_component_url_base($component_name);
		

			// warn user about notifications being disabled
			if(get_user_meta(0,'smsapi_notifications_enabled')==0){
				$msg="<div>Note: You currently have SMSapi notifications disabled.  <a href='notifymethods.php#tab-custom-smsapi'>Change settings</a>.</div>";
				$output.=get_message_text(true,false,$msg);
				}

			$output.='


	<div class="sectionTitle">SMSapi Message Settings</div>
	
	<p>Specify the format of the SMSapi messages you want to receive.</p>
	
	<p><b>NOTE:</b> The maximum length of SMSapi text messages is 160 characters.<br />Messages longer than that will be broken up into multiple messages.</p>
	
	<table>

	<tr>
	<td valign="top">
	<label>Host Alert Message:</label><br class="nobr" />
	</td>
	<td>
<textarea name="smsapi_notifications_host_message" rows="4" cols="64">'.htmlentities($smsapi_notifications_host_message).'
</textarea>
	<br class="nobr" />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>Service Alert Message:</label><br class="nobr" />
	</td>
	<td>
<textarea name="smsapi_notifications_service_message" rows="4" cols="65">'.htmlentities($smsapi_notifications_service_message).'
</textarea>
	<br class="nobr" />
	</td>
	</tr>

	</table>
			';
		
			break;
			
		case NOTIFICATIONMETHOD_MODE_SETMESSAGEFORMAT:
			// newly submitted values
			$smsapi_notifications_host_message=grab_array_var($inargs,"smsapi_notifications_host_message","");
			$smsapi_notifications_service_message=grab_array_var($inargs,"smsapi_notifications_service_message","");
			
			// save options
			set_user_meta(0,"smsapi_notifications_host_message",$smsapi_notifications_host_message);
			set_user_meta(0,"smsapi_notifications_service_message",$smsapi_notifications_service_message);

		
			break;
			
		default:
			$output="";
			break;
		}

	return $output;
	}



///////////////////////////////////////////////////////////////////////////////////////////
//CONFIG FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////

function smsapi_component_config_func($mode="",$inargs,&$outargs,&$result){

	// initialize return code and output
	$result=0;
	$output="";
	
	$component_name="smsapi";
	
	$settings_raw=get_option("smsapi_component_options");
	if($settings_raw=="") {
		$settings=array();
	}
	else {
		$settings=unserialize($settings_raw);
	}
				
	switch($mode){
		case COMPONENT_CONFIGMODE_GETSETTINGSHTML:
			// initial values
			$enabled=grab_array_var($settings,"enabled","");
			$api=grab_array_var($settings,"api","");
			$tw_sid=grab_array_var($settings,"tw_sid","");
			$tw_token=grab_array_var($settings,"tw_token","");
			$tw_from=grab_array_var($settings,"tw_from","");
			$ca_apiid=grab_array_var($settings,"ca_apiid","");
			$ca_user=grab_array_var($settings,"ca_user","");
			$ca_pass=grab_array_var($settings,"ca_pass","");
			$ca_from=grab_array_var($settings,"ca_from","");
			
			// values passed to us
			$enabled=checkbox_binary(grab_array_var($inargs,"enabled",$enabled));

			$component_url=get_component_url_base($component_name);

	$version = smsapi_component_get_arg("version");
	$date = smsapi_component_get_arg("date");
	$copyright = smsapi_component_get_arg("copyright");

	$output='

	<a href="http://www.smsapi.com" target="_blank"><img src="'.$component_url.'/images/logo.gif"></a>
	<p>Version '.$version.' ('.$date.') - 
	'.$copyright.'</p>

	<div class="sectionTitle">Integration Settings</div>
	
	<table>

	<tr>
	<td valign="top">
	<label for="enabled">Enable Integration:</label><br class="nobr" />
	</td>
	<td>
	<input type="checkbox" class="checkbox" id="enabled" name="enabled" '.is_checked($enabled,1).'>
	<br class="nobr" />
	Enables integration between Nagios XI and SMSapi.
	</td>
	</tr>
	</table>

	<div class="sectionTitle">SMSapi Settings</div>
	
	<p>These settings relate to sending alerts from Nagios XI through SMSapi.</p>

	
	<table>

	<tr>
	<td valign="top">
	<label>SMSapi:</label><br class="nobr" />
	</td>
	<td>
	<select name="api" id="api">
	<option ' . is_selected($api, "clickatell") . ' value="clickatell">Clickatell</option>
	<option ' . is_selected($api, "twilio") . ' value="twilio">Twilio</option>
	';

	$output .= '
	</select>
	<br class="nobr" />
	Which API to Use.<br /><br />
	</td>
	</tr>
	
	</table>

	<div id="apis">
	<div id="clickatell">
	<div class="sectionTitle">Clickatell Settings</div>
	<table>

	<tr>
	<td valign="top">
	<label>API Product ID:</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="40" name="ca_apiid" id="ca_apiid" value="'.$ca_apiid.'" class="textfield" /><br class="nobr" />
	Your Clickatell API Product ID.<br /><br />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>Username:</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="40" name="ca_user" id="ca_user" value="'.$ca_user.'" class="textfield" /><br class="nobr" />
	The Clickatell username you specified when creating your Clickatell account.<br /><br />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>Password:</label><br class="nobr" />
	</td>
	<td>
<input type="password" size="40" name="ca_pass" id="ca_pass" value="'.$ca_pass.'" class="textfield" /><br class="nobr" />
	Your Clickatell password.<br /><br />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>From Number:</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="ca_from" id="ca_from" value="'.$ca_from.'" class="textfield" /><br class="nobr" />
	The phone number messages will be sent from. Leave this field blank to use Clickatell gateway assigned number.<br /><br />
	</td>
	</tr>

	</table>
	</div>

	<div id="twilio">
	<div class="sectionTitle">Twilio Settings</div>
	<table>

	<tr>
	<td valign="top">
	<label>Account SID:</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="40" name="tw_sid" id="tw_sid" value="'.$tw_sid.'" class="textfield" /><br class="nobr" />
	Your Twilio Account SID.<br /><br />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>Auth Token:</label><br class="nobr" />
	</td>
	<td>
<input type="password" size="40" name="tw_token" id="tw_token" value="'.$tw_token.'" class="textfield" /><br class="nobr" />
	Your Twilio Auth Token.<br /><br />
	</td>
	</tr>

	<tr>
	<td valign="top">
	<label>From Number:</label><br class="nobr" />
	</td>
	<td>
<input type="text" size="20" name="tw_from" id="tw_from" value="'.$tw_from.'" class="textfield" /><br class="nobr" />
	The phone number messages will be sent from. This number can be your Twilio Sandbox number.<br /><br />
	</td>
	</tr>

	</table>
	</div>
	</div> <!-- apis -->

	<script type="text/javascript">

		$("#api").change(function(){
			$("#" + this.value).show().siblings().hide();
		});

		$("#api").change();

	</script>


	';

			break;
			
		case COMPONENT_CONFIGMODE_SAVESETTINGS:
			$oldport=grab_array_var($settings,"port","8444");

			// get variables
			$enabled=checkbox_binary(grab_array_var($inargs,"enabled",""));
			$api=grab_array_var($inargs,"api","");
			$tw_sid=grab_array_var($inargs,"tw_sid","");
			$tw_token=grab_array_var($inargs,"tw_token","");
			$tw_from=grab_array_var($inargs,"tw_from","");
			$ca_apiid=grab_array_var($inargs,"ca_apiid","");
			$ca_user=grab_array_var($inargs,"ca_user","");
			$ca_pass=grab_array_var($inargs,"ca_pass","");
			$ca_from=grab_array_var($inargs,"ca_from","");

			$tw_from = preg_replace('/[^\d+]/', '', $tw_from);
			$ca_from = preg_replace('/[^\d+]/', '', $ca_from);

			// validate variables
			$errors=0;
			$errmsg=array();
			if($enabled==1){
				if ($api == 'twilio') {
					if(have_value($tw_sid)==false){
						$errmsg[$errors++]="No SID specified.";
					}
					if(have_value($tw_token)==false){
						$errmsg[$errors++]="No token specified.";
					}
				}
				else if ($api == 'clickatell') {
					if(have_value($ca_apiid)==false){
						$errmsg[$errors++]="No API Product ID specified.";
					}
					if(have_value($ca_user)==false){
						$errmsg[$errors++]="No username specified.";
					}
					if(have_value($ca_pass)==false){
						$errmsg[$errors++]="No password specified.";
					}
				}
			}
			
			// save settings
			$settings=array(
				"api" => $api,
				"tw_sid" => $tw_sid,
				"tw_token" => $tw_token,
				"tw_from" => $tw_from,
				"ca_apiid" => $ca_apiid,
				"ca_user" => $ca_user,
				"ca_pass" => $ca_pass,
				"ca_from" => $ca_from,
				"enabled" => $enabled,
				);
			set_option("smsapi_component_options",serialize($settings));

			// handle errors
			if($errors>0){
				$outargs[COMPONENT_ERROR_MESSAGES]=$errmsg;
				$result=1;
				return '';
			}
			
			// info messages
			$okmsg=$buildmsg;
			$okmsg[]="SMSapi settings updated.";
			$outargs[COMPONENT_INFO_MESSAGES]=$okmsg;
			break;
		
		default:
			break;
			
		}
		
	return $output;
	}
	
function smsapi_component_get_host_message($user_id){
	$txt=get_user_meta(0,'smsapi_notifications_host_message');
	if($txt=="")
		$txt="%host% %type% (%hoststate%) %hostoutput% Addr: %hostaddress%  Time: %datetime% Nagios URL: %xiserverurl%";
	return $txt;
	}

function smsapi_component_get_service_message($user_id){
	$txt=get_user_meta(0,'smsapi_notifications_service_message');
	if($txt=="")
		$txt="%host% / %service% %type% (%servicestate%) %serviceoutput% Time: %datetime% Nagios URL: %xiserverurl%";
	return $txt;
	}


////////////////////////////////////////////////////////////////////////
// EVENT HANDLER AND NOTIFICATION FUNCTIONS
////////////////////////////////////////////////////////////////////////

register_callback(CALLBACK_EVENT_PROCESSED,'smsapi_component_eventhandler');
//smsapi_component_eventhandler_register_callbacks();

function smsapi_component_eventhandler_register_callbacks(){

	$settings_raw=get_option("smsapi_component_options");
	if($settings_raw=="")
		$settings=array();
	else
		$settings=unserialize($settings_raw);
				
	// bail out of component is not enabled
	$enabled=grab_array_var($settings,"enabled","");
	if($enabled!=1)
		return;

	register_callback(CALLBACK_EVENT_PROCESSED,'smsapi_component_eventhandler');
	}

function smsapi_component_eventhandler($cbtype,$args){

	$settings_raw=get_option("smsapi_component_options");
	if($settings_raw=="")
		$settings=array();
	else
		$settings=unserialize($settings_raw);
				
	// bail out of component is not enabled
	$enabled=grab_array_var($settings,"enabled","");
	if($enabled!=1)
		return;

	switch($args["event_type"]){
		case EVENTTYPE_NOTIFICATION:
			smsapi_component_handle_notification_event($args);
			break;
		default:
			break;
		}
	}
	
function smsapi_component_handle_notification_event($args){

	$meta=$args["event_meta"];
	$contact=$meta["contact"];
	$nt=$meta["notification-type"];
	
	// find the XI user
	$user_id=get_user_id($contact);
	if($user_id<=0)
		return;
		
	echo "==== SMSapi Notification Handler ====\n";
		
	// bail if user has notifications disabled completely
	$notifications_enabled=get_user_meta($user_id,'enable_notifications');
	if($notifications_enabled!=1){
		echo "ERROR: User has (global) notifications disabled!\n";
		return;
		}
		
	// set user id session variable - used later in date/time, preference, etc. functions
	if(!defined("NAGIOSXI_USER_ID"))
		define("NAGIOSXI_USER_ID",$user_id);

	echo " SMSapi: CONTACT=$contact, USERID=$user_id\n";
		
	// get settings
	$smsapi_notifications_enabled=get_user_meta($user_id,"smsapi_notifications_enabled");
	$smsapi_mobile_number=get_user_meta($user_id,'smsapi_mobile_number');
	
	// not enabled for this user
	if($smsapi_notifications_enabled!=1){
		echo " SMSapi: User has SMSapi notifications disabled\n";
		return 1;
		}
	
	// don't have a mobile number
	if($smsapi_mobile_number==""){
		echo " SMSapi: User does not have a mobile number specified\n";
		return 1;
		}
		
	// get the SMSapi message
	if($meta["notification-type"]=="service"){
		$message=smsapi_component_get_service_message($user_id);
		}
	else{
		$message=smsapi_component_get_host_message($user_id);
		}
		
	echo " SMSapi: RAW MESSAGE='".$message."'\n";
		
	// process notification text (replace variables)
	$message=process_notification_text($message,$meta);
	
	// trim the message
	$message=substr($message,0,159);

	echo " SMSapi: MESSAGE='".$message."'\n";
	
	$args=array(
		"number" => $smsapi_mobile_number,
		"message" => $message,
		);

	// send the SMSapi message
	$sendresult=smsapi_component_send($args);

	echo "SMSapi RESULT=$sendresult[success]\n";

	return 0;
	}

function smsapi_component_send($args) {
	$api = smsapi_component_get_option("api", "");

	switch ($api) {
		case 'clickatell':
			$sendresult=smsapi_component_clickatell_send($args);
			break;

		case 'twilio':
			$sendresult=smsapi_component_twilio_send($args);
			break;
	}

	return $sendresult;
}

?>
